﻿#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma ModuleName=normalizer
#pragma IgorVersion=8
#pragma version=1.30
#include <Resize Controls>
#include <WaveSelectorWidget>

// GUI to aid in normalising FTIR spectra to unit thickness.
// https://www.wavemetrics.com/user/tony

// 1.20 records info in wavenotes for input and output waves
// 1.30 buttons for dividing by wavemax, wave area

static strconstant ksPackageName = normalizer
static strconstant ksPrefsFileName = acwNormalizer.bin
static constant kPrefsVersion = 050

static strconstant ksPanel = "NormPanel"
static strconstant ksFilterListbox = "ListBoxWaves"

menu "Macros"
	submenu "IR and Raman"
		"Normalise Spectra", /Q, normalizer#MakeNormalizeWavesPanel()
	end
end

menu "Analysis"
	"Normalise Spectra", /Q, normalizer#MakeNormalizeWavesPanel()
end


// ------------------ package preferences --------------------

static structure PackagePrefs
	uint16 version
	uint32 options //
	char prefix[10]
	char suffix[10]
	STRUCT Rect win // window position and size, 8 bytes
	char reserved[128 - 4 - 2 - 20 - 8]
endstructure

// set prefs structure to default values
static function PrefsSetDefaults(STRUCT PackagePrefs &prefs)
	prefs.version = kPrefsVersion
	prefs.options = 3 // 1 = multiply, 2 = divide, 3 = mm -> cm
	prefs.prefix = ""
	prefs.suffix = "_cm"
	prefs.win.left = 20
	prefs.win.top = 20
	prefs.win.right = 20 + 800
	prefs.win.bottom = 20 + 410
	int i
	for(i=0;i<(128-34);i+=1)
		prefs.reserved[i] = 0
	endfor
end

static function LoadPrefs(STRUCT PackagePrefs &prefs)
	LoadPackagePreferences/MIS=1 ksPackageName, ksPrefsFileName, 0, prefs
	if (V_flag!=0 || V_bytesRead==0 || prefs.version!=kPrefsVersion)
		PrefsSetDefaults(prefs)
	endif
end

// save window position in package prefs
static function SaveWindowPosition(string strWin)
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	// save window position
	GetWindow $strWin wsizeRM
	prefs.win.top    = v_top
	prefs.win.left   = v_left
	prefs.win.bottom = v_bottom
	prefs.win.right  = v_right
	SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
end

static function selectOptions()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	string prefix = prefs.prefix
	string suffix = prefs.suffix
	variable MMCM = prefs.options
	
	Prompt prefix, "Default Prefix"
	Prompt suffix, "Default Suffix"
	Prompt MMCM, "Normalization", Popup, "multiply by factor;divide by factor;mm -> cm;"
	DoPrompt "Normalize Settings [version " + num2str(ProcedureVersion("")) + "]", prefix, suffix, MMCM
	
	if (V_flag == 1)
		return 0
	endif
	
	int requireUpdate = cmpstr(prefs.prefix, prefix) || cmpstr(prefs.suffix, suffix)
	
	prefs.prefix  = prefix
	prefs.suffix  = suffix
	prefs.options = MMCM
	
	SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
	
	if (requireUpdate)
		ResetOutputNames()	
	endif
	
	UpdateButtons()
	
	return 0
end

static function ResetOutputNames()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	STRUCT lbWaveStructure s
	getLBwaves(s)

	s.listWave[][2] = CleanupName(prefs.prefix + s.listWave[p][0] + prefs.suffix, 0)	
	SanityCheck()	
end

static function setOptions()
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	wave/T/SDFR=getDFR() titleWave
	if (prefs.options == 3)
		titleWave = {"Input","mm","Output"}
	elseif (prefs.options == 1)
		titleWave = {"Input","Multiply by","Output"}
	else
		titleWave = {"Input","Divide by","Output"}
	endif
	ControlUpdate/A/W=NormPanel
end

static function MakeNormalizeWavesPanel()
	
	KillWindow/Z $ksPanel
	DFREF dfr = resetDFR()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
		
	NewPanel/K=1/N=$ksPanel/W=(prefs.win.left,prefs.win.top,prefs.win.right,prefs.win.bottom) as "Normalize"
	SetWindow $ksPanel hook(hCleanupHook)=Normalizer#CleanupHook
	
	DefineGuide/W=$ksPanel guideVert={FL,0.3,FR}
	DefineGuide/W=$ksPanel guideHoriz={FT,0.5,FB}
	
	makeWaveSelectorWidget() // adds listbox ksFilterListbox, notebook subwindow and button ButtonClear
	
	Button btnWaveMax win=$ksPanel, pos={180,5}, size={60,15}, Proc=normalizer#buttonFunc, fsize=10
	Button btnWaveMax win=$ksPanel, title="Wave Max", help={"Set normalizing values to wave maxima"}
	Button btnArea win=$ksPanel, pos={260,5}, size={65,15}, Proc=normalizer#buttonFunc, fsize=10
	Button btnArea win=$ksPanel, title="Wave Area", help={"Set normalizing values to wave area"}
	
	Button btnGrabWaves win=$ksPanel, pos={140,90}, size={30,30}, Proc=normalizer#buttonFunc
	Button btnGrabWaves win=$ksPanel, title=">", help={"Add selected waves"}
	Button btnReleaseWaves win=$ksPanel, pos={140,130}, size={30,30}, Proc=normalizer#buttonFunc
	Button btnReleaseWaves win=$ksPanel, title="<", help={"Remove selected waves"}
	Button btnDoIt win=$ksPanel, pos={445,230}, size={45,20}, title="Do It", Proc=normalizer#buttonFunc
	Button btnCancel win=$ksPanel, pos={380,230}, size={55,20}, title="Cancel", Proc=normalizer#buttonFunc
	Button btnSettings, win=$ksPanel, pos={478,5}, size={15,15}, Picture=normalizer#pCog, labelBack=0, title=""
	Button btnSettings, win=$ksPanel, Proc=normalizer#buttonFunc, help={"Change Settings"}, focusRing=0
	ListBox ListBoxOut, win=$ksPanel, mode=8, selwave=dfr:selWave, titlewave=dfr:titleWave, pos={178, 25}, size={315, 200}, editstyle=1
	ListBox ListBoxOut, win=$ksPanel, colorwave=dfr:colorWave, listwave=dfr:listWave, widths={150,50,150}, Proc=normalizer#listBoxFunc
	
	// resizing userdata for controls
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)=A"!!,A.!!#=+!!#@^!!#AWz!!#](Aon\"Qzzzzzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox ListBoxWaves,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button btnGrabWaves,userdata(ResizeControlsInfo)=A"!!,Fq!!#?m!!#=S!!#=Sz!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	Button btnGrabWaves,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AOp`[BmKaZzzzzzzzz"
	Button btnGrabWaves,userdata(ResizeControlsInfo)+=A"zzz!!#N3Bk1ctB6@p!AOp`[BmKaZzzzzzzzzzzz!!!"
	Button btnReleaseWaves,userdata(ResizeControlsInfo)=A"!!,Fq!!#@f!!#=S!!#=Sz!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	Button btnReleaseWaves,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AOp`[BmKaZzzzzzzzz"
	Button btnReleaseWaves,userdata(ResizeControlsInfo)+=A"zzz!!#N3Bk1ctB6@p!AOp`[BmKaZzzzzzzzzzzz!!!"
	ListBox ListBoxOut,userdata(ResizeControlsInfo)=A"!!,GB!!#=+!!#BWJ,hr-z!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#o2B4uAeB6@p!AQEA_"
	ListBox ListBoxOut,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#u:DuaGlB6@p!AOp`[BmKaZzzzzzzzz"
	ListBox ListBoxOut,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAOp`[BmKaZzzzzzzzzzzz!!!"
	Button btnDoIt,userdata(ResizeControlsInfo)=A"!!,IDJ,hrK!!#>B!!#<Xz!!#o2B4uAeB6@p!AQEA_F8u:@zzzzzzzzzzz!!#o2B4uAeB6@p!AQEA_"
	Button btnDoIt,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#?(FEDG<!-N!dAOp`[BmKaZzzzzzzzz"
	Button btnDoIt,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAOp`[BmKaZzzzzzzzzzzz!!!"
	Button btnCancel,userdata(ResizeControlsInfo)=A"!!,I$!!#Au!!#>j!!#<Xz!!#o2B4uAeB6@p!AQEA_F8u:@zzzzzzzzzzz!!#o2B4uAeB6@p!AQEA_"
	Button btnCancel,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#?(FEDG<!-N!dAOp`[BmKaZzzzzzzzz"
	Button btnCancel,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAOp`[BmKaZzzzzzzzzzzz!!!"
	Button btnClear,userdata(ResizeControlsInfo)=A"!!,Fl!!#B!!!#<(!!#<(z!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	Button btnClear,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button btnClear,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button btnSettings,userdata(ResizeControlsInfo)=A"!!,IU!!#9W!!#<(!!#<(z!!#o2B4uAeB6@p!AQEA_F8u:@zzzzzzzzzzz!!#o2B4uAeB6@p!AQEA_"
	Button btnSettings,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#u:Duafn!-N!dAOp`[BmKaZzzzzzzzz"
	Button btnSettings,userdata(ResizeControlsInfo)+=A"zzz!!#u:Duafn!-N!dAOp`[BmKaZzzzzzzzzzzz!!!"
	Button btnWaveMax,userdata(ResizeControlsInfo)=A"!!,GD!!#9W!!#?)!!#<(z!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	Button btnWaveMax,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	Button btnWaveMax,userdata(ResizeControlsInfo)+=A"zzz!!#u:Duafnzzzzzzzzzzzzzz!!!"
	Button btnArea,userdata(ResizeControlsInfo)=A"!!,H=!!#9W!!#?;!!#<(z!!#N3Bk1ctB6@p!AQEA_F8u:@zzzzzzzzzzz!!#N3Bk1ctB6@p!AQEA_"
	Button btnArea,userdata(ResizeControlsInfo)+=A"F8u:@zzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	Button btnArea,userdata(ResizeControlsInfo)+=A"zzz!!#u:Duafnzzzzzzzzzzzzzz!!!"
	
	SetWindow kwTopWin,userdata(ResizeControlsInfo)=A"!!*'\"z!!#C_!!#B;zzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzz!!!"
	SetWindow kwTopWin,userdata(ResizeControlsGuides)= "guideVert;guideHoriz;fgb;fgt;fgr;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideVert)= "NAME:guideVert;WIN:NormPanel;TYPE:User;HORIZONTAL:0;POSITION:150.00;GUIDE1:FL;GUIDE2:FR;RELPOSITION:0.3;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideHoriz)= "NAME:guideHoriz;WIN:NormPanel;TYPE:User;HORIZONTAL:1;POSITION:129.00;GUIDE1:FT;GUIDE2:FB;RELPOSITION:0.5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgb)= "NAME:fgb;WIN:NormPanel;TYPE:User;HORIZONTAL:1;POSITION:247.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-11;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgt)= "NAME:fgt;WIN:NormPanel;TYPE:User;HORIZONTAL:1;POSITION:230.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-28;"
	SetWindow kwTopWin,userdata(ResizeControlsInfofgr)= "NAME:fgr;WIN:NormPanel;TYPE:User;HORIZONTAL:0;POSITION:130.00;GUIDE1:guideVert;GUIDE2:;RELPOSITION:-20;"
	
	// resizing panel hook
	SetWindow $ksPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook
		
	UpdateButtons()
end

static structure lbWaveStructure
	wave selwave
	wave/T listwave
	wave colorwave
	wave/wave waves
endstructure

static function getLBwaves(STRUCT lbWaveStructure &s)
	DFREF dfr = getDFR()
	wave s.selwave = dfr:selwave
	wave/T s.listwave = dfr:listwave
	wave s.colorwave = dfr:colorwave
	wave/wave s.waves = dfr:wWaves
end

static function CleanupHook(STRUCT WMWinHookStruct &s)
	if (s.eventcode == 2) // window is being killed
		SaveWindowPosition(s.winName)
		KillDataFolder/Z root:Packages:Normalizer
		return 0
	endif
	return 0
end

// wrapper for buttons
static function buttonFunc(STRUCT WMButtonAction &ba)

	if (ba.eventCode != 2)
		return 0
	endif
	if (cmpstr(ba.ctrlName, "btnSettings") == 0)
		selectOptions()
		setOptions()
	elseif (cmpstr(ba.ctrlName, "btnGrabWaves") == 0)
		grabWaves()
	elseif (cmpstr(ba.ctrlName, "btnReleaseWaves") == 0)
		releaseWaves()
	elseif (cmpstr(ba.ctrlName, "btnDoIt") == 0)
		doNormalization()
	elseif (cmpstr(ba.ctrlName, "btnCancel") == 0)
		KillWindow/Z $ksPanel
	elseif (cmpstr(ba.ctrlName, "btnWaveMax") == 0)
		SetNormValues(1)
	elseif (cmpstr(ba.ctrlName, "btnArea") == 0)
		SetNormValues(0)
	endif
	return 0
end


static function SetNormValues(int linear)
	
	STRUCT lbWaveStructure s
	getLBwaves(s)
	
	if (dimsize(s.listWave, 0))
		s.listWave[][1] = num2str(linear ? wavemax(s.waves[p]) : area(s.waves[p]))
	endif
	SanityCheck()
	
	return 1	
end

static function grabWaves()

	STRUCT lbWaveStructure s
	getLBwaves(s)
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	string selectionlist = WS_SelectedObjectsList(ksPanel, ksFilterListbox)
	int numselected = ItemsInList(selectionlist)
	int i
	string strNewName
	for (i=0;i<numselected;i++)
		wave w = $StringFromList(i, selectionlist)
		if (findWaveInWave(w, s.waves) < 0)
			s.waves[numpnts(s.waves)] = {w}
			strNewName = CleanupName(prefs.prefix + NameOfWave(w) + prefs.suffix, 0)
			s.listWave[DimSize(s.listWave, 0)][] = {{NameOfWave(w)},{"0"},{strNewName}}
			s.selWave[DimSize(s.selWave, 0)][] = {{0},{2},{2}}
		endif
	endfor
	WS_ClearSelection(ksPanel, ksFilterListbox)
	SanityCheck()
	UpdateButtons()
end

static function releaseWaves()
	STRUCT lbWaveStructure s
	getLBwaves(s)
	
	MatrixOP/free numSelections = sumCols(s.selWave)
	if (numSelections[0] == 0)
		return 0
	endif
	int i = DimSize(s.selWave,0)-1
	do
		if (s.selWave[i][0])
			DeletePoints/M=0 i, 1, s.selWave, s.listWave, s.waves
		endif
		i -= 1
	while (i >= 0)
	
	if (DimSize(s.selWave, 0) == 0) // preserve dimensionality of selWave
		Redimension/N=(0,3,3) s.selWave
	endif

	UpdateButtons()
end

// this code is a mess
static function DoNormalization()
	
	STRUCT lbWaveStructure s
	getLBwaves(s)
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	int count = DimSize(s.listWave, 0)
	string strNewWave = ""
	int i
	for (i=0;i<count;i+=1)
		
		variable factor = str2num(s.listWave[i][1])
		if (factor==0 || numtype(factor)!=0)
			continue
		endif
		
		factor = prefs.options == 3 ? factor/10 : factor
		
		strNewWave = GetWavesDataFolder(s.waves[i],1) + s.listWave[i][2]
		
		if (WaveExists($strNewWave))
			DoAlert 1, "Overwrite " + s.listWave[i][2] + "?"
			if (v_flag == 2)
				continue
			endif
		endif
		wave w = s.waves[i] // $listWave[i][0]
		// record sample thickness in absorbance wave
		string strNote = note(w), strNew = ""
		if (prefs.options & 2)
			strNote = ReplaceNumberByKey("Sample thickness (cm)", strNote, factor, ":", "\r")
			note/K w, RemoveEnding(strNote, "\r")
		endif
		
		if (WaveRefsEqual($strNewWave, w))
			wave w2 = w
			sprintf strnew, "Value=%g;", factor
			strNote = ReplaceStringByKey("Normalized", strNote, strNew, ":", "\r")
			note/K w, RemoveEnding(strNote, "\r")
		else
			Duplicate/O w $strNewWave /wave=w2
			// output is forced to be non-liberal
			printf "Duplicate/O %s %s\r", GetWavesDataFolder(w, 2), strNewWave
			
			strNote = note(w2)
			strNote = RemoveByKey("Sample thickness (cm)", strNote, ":", "\r")
			// record thickness and name of absorbance wave
			sprintf strnew, "Value=%g;Original=%s;", factor, NameOfWave(w)
			strNote = ReplaceStringByKey("Normalized", strNote, strNew, ":", "\r")
			note/K w2, RemoveEnding(strNote, "\r")
		endif
		
		if (prefs.options == 1)
			w2 = w * factor
		else
			w2 = w / factor
		endif
		
		// the first PossiblyQuoteName shoudn't be needed, but printf doesn't see s.listwave points as text?
		printf "%s = %s %s %g\r", PossiblyQuoteName(s.listWave[i][2]), PossiblyQuoteName(s.listWave[i][0]), SelectString(prefs.options==1, "/", "*"), factor
	endfor
	Redimension/N=(0,-1,-1,-1) s.listWave, s.selWave, s.waves
	WS_UpdateWaveSelectorWidget(ksPanel, ksFilterListbox)
end

static function findWaveInWave(wave w, wave/wave wWaves)
	if (WaveExists(w) == 0 || numpnts(wWaves)==0)
		return -1
	endif
	int i
	int numWaves = numpnts(wWaves)
	for (i=0;i<numWaves;i++)
		if (WaveRefsEqual(w, wWaves[i]))
			return i
			break
		endif
	endfor
	return -1
end

static function updateWaveNoteFormat(wave w)
	string strNote = note(w)
	string value, originalwave, strnew
	if (!GrepString(strNote, "^Normalized:1$"))
		return 0
	endif
	
	value = StringByKey("Sample thickness (cm)", ":", "\r")
	originalwave = StringByKey("Absorbance wave", ":", "\r")
	strNote = RemoveByKey("Normalized", strNote, ":", "\r")
	strNote = RemoveByKey("Sample thickness (cm)", strNote, ":", "\r")
	strNote = RemoveByKey("Absorbance wave", strNote, ":", "\r")
	strnew = ""
	sprintf strnew, "Value=%s;Original=%s;", value, originalwave
	strNote = ReplaceStringByKey("Normalized", strNote, strNew, ":", "\r")
	note/K w, strNote
	return 1
end

static function listBoxFunc(STRUCT WMListboxAction &lba)
		
	variable row = lba.row
	variable col = lba.col
	WAVE/T/Z listWave = lba.listWave
	WAVE/Z selWave = lba.selWave
	
	switch (lba.eventCode)
		case -1: // control being killed
			break
		case 1:
			if (DimSize(selWave, 0))
				selWave[][1,2][0] = selWave & ~1
			endif
			// stop edit when cell loses focus
			// so that bg color is visible
			break
		case 6: // begin edit
			ListBox $lba.ctrlName win=$lba.win, userdata(preEdit)=listWave[row][col]
			break
		case 7: // finish edit
			if (col == 1)
				if (numtype(str2num(listWave[row][col])) == 2)
					listWave[row][col] = GetUserData(lba.win, lba.ctrlName, "preEdit")
				else
					listWave[row][col] = num2str(str2num(listWave[row][col]))
				endif
				SanityCheck()
			elseif (col == 2)
				listWave[row][col] = CleanupName(listWave[row][col],0)
				SanityCheck()
			endif
			ListBox $lba.ctrlName win=$lba.win, userdata(preEdit)=""
			break
		case 4:
		case 5:
			UpdateButtons()
			break
	endswitch

	return 0
end

static function SanityCheck()

	DFREF dfr = getDFR()
	wave/T listWave = dfr:listWave
	wave selWave = dfr:selWave
	
	int i, j
	int i_max = DimSize(listWave, 0)
	
	selWave[][1,2][%backcolors] = 0
	
	for (i=0;i<i_max;i+=1)
		for (j=i+1;j<i_max;j+=1)
			if (stringmatch(listWave[i][2], listWave[j][2]))
				selWave[i][2][%backcolors] = 1
				selWave[j][2][%backcolors] = 1
			endif
		endfor
		if (WaveExists($listWave[i][2]))
			selWave[i][2][%backcolors] = 1
		endif
		
		if (str2num(listWave[i][1]) == 0)
			selWave[i][1][%backcolors] = 1
		endif
		
		if (numtype(str2num(listWave[i][1])) != 0)
			selWave[i][1][%backcolors] = 1
		endif
		
	endfor
end

static function UpdateButtons()
	
	Button btnGrabWaves win=$ksPanel, disable=2*(WS_CountSelectedObjects(ksPanel, ksFilterListbox)==0)
	
	DFREF dfr = getDFR()
	Make/free/N=1 numSelections = 0
	numSelections = 0
	wave w = dfr:selWave
	if (DimSize(w,0))
		MatrixOP/O/free numSelections = sumCols(w)
	endif
	Button btnReleaseWaves win=$ksPanel, disable=2*(numSelections[0]==0)
	Button btnDoIt win=$ksPanel, disable=2*(DimSize(w,0)==0)
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	Button btnWaveMax win=$ksPanel, disable=(prefs.options!=2)
	Button btnArea win=$ksPanel, disable=(prefs.options!=2)
	
	
	return 1
end

#if IgorVersion() < 7
static function /wave ListToTextWave(s_list, s_seperator)
	string s_list, s_seperator
	
	Make/T/free/N=(ItemsInList(s_list, s_seperator)) w
	w = StringFromList(p, s_list, s_seperator)
	
	return w
end
#endif

static function /DF getDFR()
	DFREF dfr = root:Packages:Normalizer
	if (DataFolderRefStatus(dfr) != 1)
		return resetDFR()
	endif
	return dfr
end

static function /DF resetDFR()
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:Normalizer
	DFREF dfr=root:Packages:Normalizer
	
	Make/O/N=0/T dfr:wPaths
	Make/O/N=0/WAVE dfr:wWaves
		
	Make/O/N=(0,3)/T dfr:listWave /WAVE=listWave
	Make/O/W/U/N=(3,5,3) dfr:colorWave /WAVE=colorWave
	colorWave = {{0,0,0},{65535,0,0}}
	MatrixTranspose colorWave
	Make/O/N=(0,3,3) dfr:selWave /WAVE=selWave
	SetDimLabel 2, 1, backColors, selWave // define plane 1 as background colors
		
	Make/O/N=3/T dfr:titleWave /wave=titleWave
	titleWave = {"Input","mm","Output"}
	setOptions() // resets titleWave
	
	return dfr
end

// *** wave selector widget with filter ***
// start of code that adds real-time filtering to wave selector widget

static function makeWaveSelectorWidget()
	// wave selector widget with quick filter
	int isWin = 0
	#ifdef WINDOWS
	isWin = 1 // this will be used to fix the positioning of controls for Windows
	#endif
	
	// insert a notebook subwindow to be used for filtering lists
	DefineGuide/W=$ksPanel gV={FL,0.3,FR}
	DefineGuide/W=$ksPanel fgb={FB,-11}, fgt={FB,-28}, fgr={gV,-20}
	NewNotebook /F=1/N=nbFilter/HOST=$ksPanel/W=(10,500,5200,1000)/FG=($"",fgt,fgr,fgb)/OPTS=3
	Notebook $ksPanel+"#nbFilter" fSize=12-3*isWin, showRuler=0
	Notebook $ksPanel+"#nbFilter" spacing={1, 0, 0}
	Notebook $ksPanel+"#nbFilter" margins={0,0,1000}
	SetWindow $ksPanel+"#nbFilter" activeChildFrame=0
	ClearText(1) // sets notebook to its default appearance
	SetActiveSubwindow $ksPanel
	
	// make a Button for clearing text in notebook subwindow
	Button btnClear, win=$ksPanel, pos={135,231}, size={15,15}, title=""
	Button btnClear, win=$ksPanel, Picture=normalizer#ClearTextPicture, Proc=normalizer#filterButtonProc, disable=1
	
	// insert a listbox for wave selector widget
	// filter will apply to this listbox
	ListBox $ksFilterListbox, win=$ksPanel, pos={10,25}, size={125,200}, focusRing=0
	MakeListIntoWaveSelector(ksPanel, ksFilterListbox, content=WMWS_Waves, nameFilterProc="normalizer#filterFunc")
	WS_SetNotificationProc(ksPanel, ksFilterListbox, "normalizer#WS_NotificationProc")
	WS_OpenAFolderFully(ksPanel, ksFilterListbox, GetDataFolder(1))
		
	SetWindow $ksPanel hook(hFilterHook)=Normalizer#FilterHook
end

// function used by WaveSelectorWidget
static function filterFunc(string aName, variable contents)

	string nb = ksPanel + "#nbFilter"
	
	if (WinType(nb) != 5)
		return 1
	endif
		
	string strFilter = GetUserData(nb, "", "filter")
	
	string leafName = ParseFilePath(0, aName, ":", 1, 0)
	int vFilter = 0
	try
		vFilter = GrepString(leafName, "(?i)" + strFilter); AbortOnRTE
	catch
		int err = GetRTError(1)
	endtry
	return vFilter
end

static function ClearText(int doIt)
	if (doIt) // clear filter widget
		string nb = ksPanel + "#nbFilter"
		Notebook $nb selection={startOfFile,endofFile}, textRGB=(50000,50000,50000), text="Filter"
		Notebook $nb selection={startOfFile,startOfFile}
		Button btnClear, win=$ksPanel, disable=3
		SetWindow $nb UserData(filter) = ""
	endif
end

// intercept and deal with keyboard events in notebook subwindow
static function FilterHook(STRUCT WMWinHookStruct &s)
	
	if (s.eventcode == 4) // mousemoved
		return 0
	endif
	
	string nb = ksPanel + "#nbFilter"
		
	GetWindow/Z $ksPanel activeSW
	if (cmpstr(s_value, nb))
		return 0
	endif
	
	if (s.eventCode == 22 && cmpstr(s.WinName, nb)==0) // don't allow scrolling
		return 1
	endif
	
	string strFilter = GetUserData(nb, "", "filter")
	int vLength = strlen(strFilter)
	
	if (s.eventcode==3 && vLength==0) // mousedown
		return 1 // don't allow mousedown when we have 'filter' displayed in nb
	endif
		
	if (s.eventcode == 10) // menu
		strswitch(s.menuItem)
			case "Paste":
				string strScrap = GetScrapText()
				strScrap = ReplaceString("\r", strScrap, "")
				strScrap = ReplaceString("\n", strScrap, "")
				strScrap = ReplaceString("\t", strScrap, "")
				
				GetSelection Notebook, $nb, 1 // get current position in notebook
				if (vLength == 0)
					Notebook $nb selection={startOfFile,endofFile}, text=strScrap
				else
					Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=strScrap
				endif
				vLength += strlen(strScrap) - abs(V_endPos - V_startPos)
				s.eventcode = 11
				// pretend this was a keyboard event to allow execution to continue
				break
			case "Cut":
				GetSelection Notebook, $nb, 3 // get current position in notebook
				PutScrapText s_selection
				Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=""
				vLength -= strlen(s_selection)
				s.eventcode = 11
				break
			case "Clear":
				GetSelection Notebook, $nb, 3 // get current position in notebook
				Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text="" // clear text
				vLength -= strlen(s_selection)
				s.eventcode = 11
				break
		endswitch
		Button btnClear, win=$ksPanel, disable=3*(vLength == 0)
		ClearText((vLength == 0))
	endif
				
	if (s.eventcode != 11)
		return 0
	endif
	
	if (vLength == 0) // Remove "Filter" text before starting to deal with keyboard activity
		Notebook $nb selection={startOfFile,endofFile}, text=""
	endif
	
	// deal with some non-printing characters
	switch(s.keycode)
		case 9:	// tab: jump to end
		case 3:
		case 13: // enter or return: jump to end
			Notebook $nb selection={endOfFile,endofFile}
			break
		case 28: // left arrow
		case 29: // right arrow
			ClearText((vLength == 0)); return (vLength == 0)
		case 8:
		case 127: // delete or forward delete
			GetSelection Notebook, $nb, 1
			if (V_startPos == V_endPos)
				V_startPos -= (s.keycode == 8)
				V_endPos += (s.keycode == 127)
			endif
			V_startPos = min(vLength,V_startPos); V_endPos = min(vLength,V_endPos)
			V_startPos = max(0, V_startPos); V_endPos = max(0, V_endPos)
			Notebook $nb selection={(0,V_startPos),(0,V_endPos)}, text=""
			vLength -= abs(V_endPos - V_startPos)
			break
	endswitch
		
	// find and save current position
	GetSelection Notebook, $nb, 1
	int selEnd = V_endPos
		
	if (strlen(s.keyText) == 1) // a one-byte printing character
		// insert character into current selection
		Notebook $nb text=s.keyText, textRGB=(0,0,0)
		vLength += 1 - abs(V_endPos - V_startPos)
		// find out where we want to leave cursor
		GetSelection Notebook, $nb, 1
		selEnd = V_endPos
	endif
	
	// select and format text
	Notebook $nb selection={startOfFile,endOfFile}, textRGB=(0,0,0)
	// put text into global filter string
	GetSelection Notebook, $nb, 3
	strFilter = s_selection
	Notebook $nb selection={(0,selEnd),(0,selEnd)}, findText={"",1}
	
	Button btnClear, win=$ksPanel, disable=3*(vLength==0)
	ClearText((vLength == 0))
		
	SetWindow $nb UserData(filter) = strFilter
	
	WS_UpdateWaveSelectorWidget(ksPanel, ksFilterListbox)
	
	return 1 // tell Igor we've handled all keyboard events
end

static function filterButtonProc(STRUCT WMButtonAction &s)
	if (s.eventCode != 2)
		return 0
	endif
	ClearText(1)
	WS_UpdateWaveSelectorWidget(ksPanel, ksFilterListbox)
end

// PNG: width= 90, height= 30
static Picture ClearTextPicture
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U"$&q@5u_NKm@(_/W^%DU?TFO*%Pm('G1+?)0-OWfgsSqYDhC]>ST`Z)0"D)K8@Ncp@>C,GnA#([A
	Jb0q`hu`4_P;#bpi`?T]j@medQ0%eKjbh8pO.^'LcCD,L*6P)3#odh%+r"J\$n:)LVlTrTIOm/oL'r
	#E&ce=k6Fiu8aXm1/:;hm?p#L^qI6J;D8?ZBMB_D14&ANkg9GMLR*Xs"/?@4VWUdJ,1MBB0[bECn33
	KZ1__A<"/u9(o<Sf@<$^stNom5GmA@5SIJ$^\D=(p8G;&!HNh)6lgYLfW6>#jE3aT_'W?L>Xr73'A#
	m<7:2<I)2%%Jk$'i-7@>Ml+rPk4?-&B7ph6*MjH9&DV+Uo=D(4f6)f(Z9"SdCXSlj^V?0?8][X1#pG
	[0-Dbk^rVg[Rc^PBH/8U_8QFCWdi&3#DT?k^_gU>S_]F^9g'7.>5F'hcYV%X?$[g4KPRF0=NW^$Z(L
	G'1aoAKLpqS!ei0kB29iHZJgJ(_`LbUX%/C@J6!+"mVIs6V)A,gbdt$K*g_X+Um(2\?XI=m'*tR%i"
	,kQIh51]UETI+HBA)DV:t/sl4<N*#^^=N.<B%00/$P>lNVlic"'Jc$p^ou^SLA\BS?`$Jla/$38;!#
	Q+K;T6T_?\3*d?$+27Ri'PYY-u]-gEMR^<d.ElNUY$#A@tX-ULd\IU&bfX]^T)a;<u7HgR!i2]GBpt
	SiZ1;JHl$jf3!k*jJlX$(ZroR:&!&8D[<-`g,)N96+6gSFVi$$Gr%h:1ioG%bZgmgbcp;2_&rF/[l"
	Qr^V@O-"j&UsEk)HgI'9`W31Wh"3^O,KrI/W'chm_@T!!^1"Y*Hknod`FiW&N@PIluYQdKILa)RK=W
	Ub4(Y)ao_5hG\K-+^73&AnBNQ+'D,6!KY/`F@6)`,V<qS#*-t?,F98]@h"8Y7Kj.%``Q=h4.L(m=Nd
	,%6Vs`ptRkJNBdbpk]$\>hR4"[5SF8$^:q=W([+TB`,%?4h7'ET[Y6F!KJ3fH"9BpILuUI#GoI.rl(
	_DAn[OiS_GkcL7QT`\p%Sos;F.W#_'g]3!!!!j78?7R6=>B
	ASCII85End
end

// end of filter code

static function WS_NotificationProc(string SelectedItem, variable EventCode)
	switch (eventCode)
		case 3: // double click
			if (strlen(SelectedItem))
				STRUCT WMButtonAction s
				s.ctrlName = "btnGrabWaves"
				s.eventCode = 2
				ButtonFunc(s)
			endif
			break
		case 4:
		case 5:
			UpdateButtons()
	endswitch
	return 0
end

// PNG: width= 90, height= 30
static Picture pCog
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U%adj95u_NKeXCo&'5*oW5;QerS2ddE(FP;'6Ql#oBiuS1dW/'TBu9uWoJd,<?@,Ku[OT_*C6>N&V
	5fgKO-Z!0m]BHZFE$kSY/b+).^UIV80KE-KEW"FQ9_,eF^bT7+UPEE3;1R;IlL+VJZGMtc/)57aYSB
	X^5kLMq7Sp9q0YBG@U214ph]9b)&eA4@(!#Ip%g0/F0_UH>\E'7M.=ui]mI6`m'G"T)B'OA[IVmQDm
	=?Eat`-Qd[5bR1c#F]`cT(Se*XgJen[DI=<Y8YF7N!d-X+iY2pq;SVDt;siWS_bs$!E]WT^pRhs]aH
	M%csq\akmJ4tMo<HGU1WqT,5cib%XrHM`>*]/&U:](RQ7QDj\u&#cMNi8>`?8;-?rCcXX>+1^gW10F
	n!h-2G-M;R\a+TD2op'?`R]"%W!DpJmO],i(@-(/F?,;L7Vkt%YOa,f\8+CV@lHVOEMgMZPnh$6>!V
	MTYBm^8f[O,?Z$2MnH6.T'JWSM4HtSissRo-):4d:ecoe5Tn^(gUEQm+o94@;L(/[91%aXk:!pP;mm
	\kh$s.7qbe%=-p1eBtDs*CHp:+$CUY\0A,jM0:4J2&pY-HWBG?nb`"BE/M-#*)+E?I*C/(r;J]APNh
	3!Ea<(u)`o?0R`ma=QV<n?GV/s3:I0Wf2_M0p@:__T%OEl+sL@10K8&ViQgR(0Q3qMLYA':/iba:,;
	]Y$@ACMV&9b[fD4A`Vq5+A!37VD0na`;0#fWNWKq#f5N>Mt)$S['[2:?=(p2$Q$$NX_cXoJ`iVOcHm
	Rb+"_b#*b4@tp)Xq9r*1_<^IVlpMJ=kE>MhiHa2]]q9<d*4(lA_8$4ej2NM5Z!#`oc=+Ttk-]%D5"O
	Yiu,o$V/I<=@2fN3Ds,PNfIEnqn6C?^[OYDs4q2k*s6TFu+@1>SKUmdko@B5>Pp)-]8`l_Ig,/1c.T
	K'Z+asa)qDc*mqZAKmijlOd;;&H$MEMWY1:\q<G#aaNVlho?TWCGL35!G658MH$RpQ,/[:S#==eP-@
	T+s%'h-7&.0)\eW6j@1gNW'FYlgRIid1g1dP0.MtL)"o@*4A+B&XU\9bRSJWg?B!keI%b6T7FS'?W(
	@7j-a-n[,Adkh%U((6"oN9G"iRV$rmb0"2lqXk08!N&V_b13Oo*tc)h=[L]E@W<ihr:]%Dbs-*cCbc
	T^`<b81D1(d_gue7JX)rMl-Q!ag!0.a4mbCL5'MQH2X<p3`1nA#69QNiWDnN[J^:kIm`JPCXo#W8lo
	?KFN_dOMp#7s\i!;q:1Vb`q^Za5j'0Sg8ALVn\tm:OM*.G/IF;=K4S+O/0U]^`u\O,i'69Y*0'f,SH
	:Kp)mH3.EQ3hDf%?f;W[LbJbR7R5Lb$8U4I7,[8ZM7fU7H5(>6BlSlr1cG6"263q!"YH.\`>aLYN(!
	e3hZYm666$Mq_c(_GHO?%CE(rnI-UV=I6M\e$%CXt$`9q"IB8d`/4e)0&Dcf`43oobf6MqdV?d>\73
	X/dI(2jZ+#[Nt'sQg5KRFl_^rEVX>cZ5h3g1gR#*IhTZf+KqmsVrW[`UcE6>MM*N0fMcT[S!:h'=ju
	EaL@5>OfU]WXd+d/JISLYCQZ_BPkB$IiAYUBq1l^ecC4a8EYJ'WJ,pak5V59k6$F23m\(d</D&W$.c
	&:n64N(\`j0%h;m7iB=j)(R^kh8BUCfn<.JP[26K1F7\>0JOc56jWCOX(6k=i$m^+A%<G5QZ.i$e8]
	01cU/k$R?&@c[1P\L>tK[OkSMmQ7lT_puI&4&#-'R99q+p;3\Sn`Ic2G%kDj/"N6oB<E0?Z6Kl"@,Y
	?4P5G,Nu\q^LTs(Vj(g3YCoZEl;W9!T)>ePG,S!10n_Y\rP(FBr90.J3;jVP^MYp<-ML6>(=)*bFsB
	&uWX-VWg'<P8G@$+\A?J1@G'F,/Z.<q/;.,=8I?;gFp>>;IjEQPE_:7`.R+3bElA@DB6<k@ksJ9lg(
	=CVM=g<G(^E#SiiFHZ8.qF-^ppkE&\[U*_);<'Lfk*Fq]^#W339=Q'IWpj"KgUWU]1,ETW<&G^OtOH
	MkGSX1r0@A!=Gd&>m2;$s(nF8b!K?8R`eZjO8V38_I$<1AcnP!)B*`T#3.X=hE[s8PMoFf*/H2@32u
	N'Len`GUk?n:K*@=9gMMi1ES9gF7flQcCD`2n^,h:`S5=GNQ^G#@^/a:?]W`PV50n4Xue>QVk8E1=]
	lWKB?uV(SiXjL_hVC,G-+W-bHd)[C`^u(BPM:VV58ltJcZ8d$CEhp-Ek)Qb.'^'f)4eT`-]7*:O[1>
	WO?>HRRZ3%&Aum4gNS.jU=.^S*B#H\'0H3Z)tG>O;)WW1_IO09[>@PI4Y3R7J0]^!UgQhj+u1-,O_#
	4q7aj2?Y5.V]ph=D*J`g8?n%JH:8P)K%MLrfVTs(X1]A:d+mFtdNBG"";'8siHNZC4&bKJq`%mNb7r
	SW;=`2-+n=L)HDOsFHoS$CX_6m<3W758iRSt7"9?7u`s%5^"&NsfV]&fKiR),nod"F>!-ZZj257RH'
	BCpkU2>pgE:BYW?=m,Fa:(R[)N0g+8U_d1?h6?87.J?1.S8QM+N.?c/5H^[K9Qq/KSe*09PFi*)kOs
	Cpa8151hB![K\`b9:/2t#`0h)TQGGW^_mOCaj@jCA@uU*q95,uIW@7!X&<O\"P/_=YGg"!\PP(fcMs
	XX8]B01I/5GVm/oHal.qLkA^e?r`*cq5<6cMYpEN]i/%/Vl(p'^fIMd8C\oHLnT1!)7oQm]mKeID*1
	Nn<=:2#ZkEe#YpHHH,[1j)T%7I4;@/)4cu_Q1)PbZM:[@i"UEI%;r>p03XoVZ_RkU=+$'7#=USG/bL
	8-+m<=>h,iqN=A8kNQ3E0-ce+TlUO7L$\:&5CW07\^Y5(=Lpj3bn5fXf]+hD?H]7Wq?&[-c"7hNK0#
	/)B'Mj<HT8ma3CurC,*bh\'aHR7Z[QbX-ZmAk800m):lpO:?P+(!;TY'R\j"AmjWGZ^'4n^bf?W3J@
	>&NBK1HqWkVhSJ@2:%U$9,hq:d,G1cCmI-TcrP\J*Ut[2@6?BcK3XN6]^DH?sm>]m;PWk0+t]M3*pb
	_i5ToaNr1&dko4ib1O7G-^#a3R58IWd+6c;6ULrU<E06&]A7B"H::^+p=jM"Cht@E-\k9W-F%RN7[f
	g9`s&hll-^kfa*7KZfMd!Yc("^(?tb@(G_h8BF>IEA!<RhTlND,!db&r!Y3m>2$M&6dHhJmnR;"(TN
	7k9deNFM:rt_%M:_\bI5MO2h<A0N#R:ZSrC"&ps\iY*%&:=-;@IrX+"G9!l_&sOI?=_'7)$hsk)[Og
	CfZ9V$@mNB]AS#G_>V6^Z_/)$iF?19\*_+U8'Lh!@O$@74\ogtP<K2K,l#,B;0e6NJ*#oS35C1Gqa?
	[/#EMYbdp\'g083t^Hm.OC']=?TCh[+\@'/CD^$lb9k>s0TnKIaqh4fI!&lDq*\4*U*,*??/2AnId;
	.P@%q^Y_gV7L#<Y@CP!Nm,EKn=&i8<r@!PTa5]H_PQ](fBqn;iMUE,Pl5E-<#!*W9G0Gifi6\]*<o4
	FnfrX,WF^_\F$1nF]^fU-bLO!=bm%5lG.k3$IWMqUu'c@l,R*B4I#7$5RGsBAo;S"q!W&r%8C2/"PK
	bsa<4<J73edT0;DW#W4)GM@uFDONL"9R+_N^([T!(k&'aJc*VLHV&^R;!(_#4Z$g7@#[rnEd4b][m6
	"s2Cf6FX#Yth+![-Bc9;DCc35!#ZOe]>R5l%A3s9r*"E3cZ^HAq!@!X3ZLR*.A7oQ8om.\p[kV)RYJ
	YS_VJ,ke(!91A):`eg`H3<A0s%C//=2RBq,pC2:S]*\P@U_Oa*3.T[g!Hf.uI$^Z574:Ig+a&Rh&b)
	g:i!IBPVCY]Y$?Mc-nM/==coe'#A=jP*M;$C2,4.LP+[KAA[:ZiG]VW6ipmf;5gRtUogbYmG#,M=Y+
	23\;kLm>:;.qMlKqn)uClJ![.i',6Vo?VRu"<5C39QHia.rIUXNcq/492/E<t4:q=5j]#0#BT^2C8R
	r=7NOaA&EGCimE'I"(o(b72sE7cRPmWBnj]tHh/;(=(Hs(iTH$*GN/REC6P4"-OQ)1Z*S9OlNXqYG,
	/qFo"%Xt5WcH)DMDo_@'gn2mp\l)\(b!Y0Pa!2n,Nj%-R@Yjn?WT$E#t(FUbj=.R08ON,:0qYL%:/M
	0]<Q1)2)G':0@s*h8ZZ<4MLPu4'A3d&SI6l9j+cCI%0ltDh?G(&13<0N\Q=?t??;p9Q8$59_n3Rli=
	5b`N"1`$#>;[JNr+$!*^fli%qGlHBaGd#r_gndbH0<a<pR0sE5L0:j)I_t)\EH/7Wqt8QJMd<r<&WK
	8J3cuoH9hij#22_bS-?/1q+bUC@(DjDc_1Dg*LCYK([C$_m"OB=44C54XF6CiRHM)#JSik-Qi#lgdX
	C:AAV;n0(-%L_0m!T,"d5MVG@HhTKZ87K.p%WH^Xh3kCpXcTY<@p]%p!H!PcGm8Ma`dWI-i:%O`.@A
	\EMhGlp>Rk7O<G2m`*r,h[u\8;4r,bUaQp%EDN*J]D4B1hFXuppq^tpModARV5%<QlNN?L%hAEkIlW
	/#`^]Bs#-d.f-97RH2#l<o@ZXZ&T?iRH-<%N9SU+'so7AAgW2mil0fWaM)A,6@)4Rp@WFC0@Y,uIN:
	5uCJkMPAJFd6VVd/UR6[rI=?0fVgq/kI,s^(YARDN54WD\PD#"bV<C5XKS<`]Fql#m@)ul]O#Nn]-2
	[lB);<HM,sPARkJs:;d4_S!/nh?qGe7@I:AsnMi7DjM_D$2XW>fsY^ZQI8$;`nm!f"mg^^mq'BNs/!
	!!!j78?7R6=>B
	ASCII85End
end

#if (exists("ProcedureVersion") != 3)
// replicates ProcedureVersion function for older versions of Igor
static function ProcedureVersion(string win)
	variable noversion = 0 // default value when no version is found
	if (strlen(win) == 0)
		string strStack = GetRTStackInfo(3)
		win = StringFromList(ItemsInList(strStack, ",") - 2, strStack, ",")
		string IM = " [" + GetIndependentModuleName() + "]"
	endif
	
	wave/T ProcText = ListToTextWave(ProcedureText("", 0, win + IM), "\r")
	
	variable version
	Grep/Q/E="(?i)^#pragma[\s]*version[\s]*=" /LIST/Z ProcText
	s_value = LowerStr(TrimString(s_value, 1))
	sscanf s_value, "#pragma version = %f", version

	if (V_flag!=1 || version<=0)
		return noversion
	endif
	return version
end
#endif
